"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.QuestStatus = void 0;
const JSONTemplate_1 = require("modloader64_api/JSONTemplate");
const FlagManager_1 = require("modloader64_api/FlagManager");
class QuestStatus extends JSONTemplate_1.JSONTemplate {
    emulator;
    songFlags;
    songFlagsAddr = 0x803C4CC5;
    constructor(emu, core) {
        super();
        this.emulator = emu;
        this.songFlags = new FlagManager_1.FlagManager(emu, this.songFlagsAddr);
    }
    jsonFields = [
        "hasTunic",
        "braceletEquip",
        "triforce",
        "pearls",
        "max_hp",
        "max_mp",
        "bracelet",
        "pirate_charm",
        "hero_charm",
        "owned_charts",
        "opened_charts",
        "completed_charts",
        "sectors",
        "deciphered_triforce",
        "songs",
        "tingle_statues",
    ];
    get max_hp() {
        return this.emulator.rdramRead16(0x803C4C08);
    }
    set max_hp(flag) {
        this.emulator.rdramWrite16(0x803C4C08, flag);
    }
    get max_mp() {
        return this.emulator.rdramRead8(0x803C4C1B);
    }
    set max_mp(flag) {
        this.emulator.rdramWrite8(0x803C4C1B, flag);
    }
    get current_hp() {
        return this.emulator.rdramRead16(0x803C4C0A);
    }
    set current_hp(flag) {
        this.emulator.rdramWrite16(0x803C4C0A, flag);
    }
    get current_mp() {
        return this.emulator.rdramRead8(0x803C4C1C);
    }
    set current_mp(flag) {
        this.emulator.rdramWrite8(0x803C4C1C, flag);
    }
    //Bitfields 
    get songs() {
        return this.emulator.rdramReadBuffer(0x803C4CC5, 0x1);
    }
    set songs(flag) {
        this.emulator.rdramWriteBuffer(0x803C4CC5, flag);
    }
    get hasTunic() {
        return this.emulator.rdramReadBit8(0x803C5256, 0);
    }
    set hasTunic(flag) {
        this.emulator.rdramWriteBit8(0x803C5256, 0, flag);
    }
    get bracelet() {
        return this.emulator.rdramRead8(0x803C4CBE);
    }
    set bracelet(flag) {
        this.emulator.rdramWrite8(0x803C4CBE, flag);
        if (flag !== 0)
            this.emulator.rdramWrite8(0x803C4C18, 0x28);
    }
    get currentWallet() {
        return this.emulator.rdramRead8(0x803C4C1A);
    }
    set currentWallet(flag) {
        this.emulator.rdramWrite8(0x803C4C1A, flag);
    }
    get pirate_charm() {
        return this.emulator.rdramReadBuffer(0x803C4CBF, 0x1);
    }
    set pirate_charm(flag) {
        this.emulator.rdramWriteBuffer(0x803C4CBF, flag);
    }
    get hero_charm() {
        return this.emulator.rdramReadBit8(0x803C4CC0, 7);
    }
    set hero_charm(flag) {
        this.emulator.rdramWriteBit8(0x803C4CC0, 7, flag);
    }
    /*
    0 - Wind's Requiem
    1 - Ballad of Gales
    2 - Command Melody
    3 - Earth God's Lyric
    4 - Wind God's Aria
    5 - Song of Passing
    */
    windsRequiemFlag = new FlagManager_1.Flag(0x0, 0);
    get windsRequiem() {
        return this.songFlags.isFlagSet(this.windsRequiemFlag);
    }
    set windsRequiem(bool) {
        this.songFlags.setFlag(this.windsRequiemFlag, bool);
    }
    balladGalesFlag = new FlagManager_1.Flag(0x0, 1);
    get balladGales() {
        return this.songFlags.isFlagSet(this.balladGalesFlag);
    }
    set balladGales(bool) {
        this.songFlags.setFlag(this.balladGalesFlag, bool);
    }
    commandMelodyFlag = new FlagManager_1.Flag(0x0, 2);
    get commandMelody() {
        return this.songFlags.isFlagSet(this.commandMelodyFlag);
    }
    set commandMelody(bool) {
        this.songFlags.setFlag(this.commandMelodyFlag, bool);
    }
    earthLyricFlag = new FlagManager_1.Flag(0x0, 3);
    get earthLyric() {
        return this.songFlags.isFlagSet(this.earthLyricFlag);
    }
    set earthLyric(bool) {
        this.songFlags.setFlag(this.earthLyricFlag, bool);
    }
    windAriaFlag = new FlagManager_1.Flag(0x0, 4);
    get windAria() {
        return this.songFlags.isFlagSet(this.windAriaFlag);
    }
    set windAria(bool) {
        this.songFlags.setFlag(this.windAriaFlag, bool);
    }
    songPassingFlag = new FlagManager_1.Flag(0x0, 5);
    get songPassing() {
        return this.songFlags.isFlagSet(this.songPassingFlag);
    }
    set songPassing(bool) {
        this.songFlags.setFlag(this.songPassingFlag, bool);
    }
    get triforce() {
        return this.emulator.rdramReadBuffer(0x803C4CC6, 0x1);
    }
    set triforce(flag) {
        this.emulator.rdramWriteBuffer(0x803C4CC6, flag);
    }
    /*    803C4CC7,1 - Bitfield of which pearls you own ("symbols").
    0 - Nayru's Pearl
    1 - Din's Pearl
    2 - Farore's Pearl*/
    get pearls() {
        return this.emulator.rdramReadBuffer(0x803C4CC7, 0x1);
    }
    set pearls(flag) {
        this.emulator.rdramWriteBuffer(0x803C4CC7, flag);
    }
    get owned_charts() {
        return this.emulator.rdramReadBuffer(0x803C4CDC, 0xF);
    }
    set owned_charts(flag) {
        this.emulator.rdramWriteBuffer(0x803C4CDC, flag);
    }
    get opened_charts() {
        return this.emulator.rdramReadBuffer(0x803C4CEC, 0xF);
    }
    set opened_charts(flag) {
        this.emulator.rdramWriteBuffer(0x803C4CEC, flag);
    }
    get completed_charts() {
        return this.emulator.rdramReadBuffer(0x803C4CFC, 0xF);
    }
    set completed_charts(flag) {
        this.emulator.rdramWriteBuffer(0x803C4CFC, flag);
    }
    get sectors() {
        return this.emulator.rdramReadBuffer(0x803C4D0C, 0x30);
    }
    set sectors(flag) {
        this.emulator.rdramWriteBuffer(0x803C4D0C, flag);
    }
    get deciphered_triforce() {
        return this.emulator.rdramReadBuffer(0x803C4D4D, 0x1);
    }
    set deciphered_triforce(flag) {
        this.emulator.rdramWriteBuffer(0x803C4D4D, flag);
    }
    get tingle_statues() {
        return this.emulator.rdramReadBuffer(0x803c5296, 0x1);
    }
    set tingle_statues(flag) {
        this.emulator.rdramWriteBuffer(0x803c5296, flag);
    }
}
exports.QuestStatus = QuestStatus;
//# sourceMappingURL=QuestStatus.js.map