"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ShieldsEquipment = void 0;
const JSONTemplate_1 = require("modloader64_api/JSONTemplate");
class ShieldsEquipment extends JSONTemplate_1.JSONTemplate {
    emulator;
    core;
    jsonFields = ['shieldLevel'];
    constructor(emulator, core) {
        super();
        this.emulator = emulator;
        this.core = core;
    }
    get shieldLevel() {
        let bits = this.emulator.rdramReadBits8(0x803C4CBD); //Shield bitfield
        if (bits[6 /* ShieldBitMap.MIRROR */] === 1) {
            return 2 /* Shield.MIRROR */;
        }
        else if (bits[7 /* ShieldBitMap.HERO */] === 1) {
            return 1 /* Shield.HERO */;
        }
        else {
            return 0 /* Shield.NONE */;
        }
    }
    /*
      803C4C17,1 - Currently equipped shield ID.
        3B - Hero's Shield
        3C - Mirror Shield
      */
    set shieldLevel(level) {
        let bits = this.emulator.rdramReadBits8(0x803C4CBD);
        switch (level) {
            case 0 /* Shield.NONE */:
                bits[7 /* ShieldBitMap.HERO */] = 0;
                bits[6 /* ShieldBitMap.MIRROR */] = 0;
                this.emulator.rdramWrite8(0x803C4C17, 0xFF);
                break;
            case 1 /* Shield.HERO */:
                bits[7 /* ShieldBitMap.HERO */] = 1;
                bits[6 /* ShieldBitMap.MIRROR */] = 0;
                this.emulator.rdramWrite8(0x803C4C17, 0x3B);
                break;
            case 2 /* Shield.MIRROR */:
                bits[7 /* ShieldBitMap.HERO */] = 0;
                bits[6 /* ShieldBitMap.MIRROR */] = 1;
                this.emulator.rdramWrite8(0x803C4C17, 0x3C);
                break;
        }
        this.emulator.rdramWriteBits8(0x803C4CBD, bits);
    }
}
exports.ShieldsEquipment = ShieldsEquipment;
//# sourceMappingURL=ShieldsEquipment.js.map