"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StageInfo = void 0;
const JSONTemplate_1 = require("modloader64_api/JSONTemplate");
class StageInfo extends JSONTemplate_1.JSONTemplate {
    emulator;
    stageInfoAddr = 0x803C4F88;
    stageID;
    jsonFields = [
        "chests",
        "switches",
        "items",
        "rooms",
        "keys",
        "map",
        "compass",
        "bigKey",
        "bossKilled",
        "heartTaken",
        "bossIntroWatched",
    ];
    constructor(emu, stageID, instance) {
        super();
        this.emulator = emu;
        this.stageID = stageID;
        if (instance !== undefined) {
            this.stageInfoAddr = instance;
        }
    }
    get chests() {
        return this.emulator.rdramReadBuffer(this.stageInfoAddr + (0x24 * this.stageID), 0x4);
    }
    set chests(flag) {
        this.emulator.rdramWriteBuffer(this.stageInfoAddr + (0x24 * this.stageID), flag);
    }
    get switches() {
        return this.emulator.rdramReadBuffer(this.stageInfoAddr + (0x24 * this.stageID) + 0x4, 0x10);
    }
    set switches(flag) {
        this.emulator.rdramWriteBuffer(this.stageInfoAddr + (0x24 * this.stageID) + 0x4, flag);
    }
    get items() {
        return this.emulator.rdramReadBuffer(this.stageInfoAddr + (0x24 * this.stageID) + 0x14, 0x4);
    }
    set items(flag) {
        this.emulator.rdramWriteBuffer(this.stageInfoAddr + (0x24 * this.stageID) + 0x14, flag);
    }
    get rooms() {
        return this.emulator.rdramReadBuffer(this.stageInfoAddr + (0x24 * this.stageID) + 0x18, 0x8);
    }
    set rooms(flag) {
        this.emulator.rdramWriteBuffer(this.stageInfoAddr + (0x24 * this.stageID) + 0x18, flag);
    }
    get keys() {
        return this.emulator.rdramRead8(this.stageInfoAddr + (0x24 * this.stageID) + 0x20);
    }
    set keys(flag) {
        this.emulator.rdramWrite8(this.stageInfoAddr + (0x24 * this.stageID) + 0x20, flag);
    }
    get map() {
        let bit = this.emulator.rdramReadBit8(this.stageInfoAddr + (0x24 * this.stageID) + 0x21, 7);
        return bit;
    }
    set map(flag) {
        this.emulator.rdramWriteBit8(this.stageInfoAddr + (0x24 * this.stageID) + 0x21, 7, flag);
    }
    get compass() {
        let bit = this.emulator.rdramReadBit8(this.stageInfoAddr + (0x24 * this.stageID) + 0x21, 6);
        return bit;
    }
    set compass(flag) {
        this.emulator.rdramWriteBit8(this.stageInfoAddr + (0x24 * this.stageID) + 0x21, 6, flag);
    }
    get bigKey() {
        let bit = this.emulator.rdramReadBit8(this.stageInfoAddr + (0x24 * this.stageID) + 0x21, 5);
        return bit;
    }
    set bigKey(flag) {
        this.emulator.rdramWriteBit8(this.stageInfoAddr + (0x24 * this.stageID) + 0x21, 5, flag);
    }
    get bossKilled() {
        let bit = this.emulator.rdramReadBit8(this.stageInfoAddr + (0x24 * this.stageID) + 0x21, 4);
        return bit;
    }
    set bossKilled(flag) {
        this.emulator.rdramWriteBit8(this.stageInfoAddr + (0x24 * this.stageID) + 0x21, 4, flag);
    }
    get heartTaken() {
        let bit = this.emulator.rdramReadBit8(this.stageInfoAddr + (0x24 * this.stageID) + 0x21, 3);
        return bit;
    }
    set heartTaken(flag) {
        this.emulator.rdramWriteBit8(this.stageInfoAddr + (0x24 * this.stageID) + 0x21, 3, flag);
    }
    get bossIntroWatched() {
        let bit = this.emulator.rdramReadBit8(this.stageInfoAddr + (0x24 * this.stageID) + 0x21, 2);
        return bit;
    }
    set bossIntroWatched(flag) {
        this.emulator.rdramWriteBit8(this.stageInfoAddr + (0x24 * this.stageID) + 0x21, 2, flag);
    }
}
exports.StageInfo = StageInfo;
//# sourceMappingURL=StageInfo.js.map