/// <reference types="node" />
import { Heap } from 'modloader64_api/heap';
import { ICore } from 'modloader64_api/IModLoaderAPI';
export declare const enum Sword {
    NONE = 0,
    Hero = 1,
    Master = 2,
    MasterHalf = 3,
    MasterFull = 4
}
export interface ISwords {
    swordLevel: Sword;
}
export declare const enum Shield {
    NONE = 0,
    HERO = 1,
    MIRROR = 2
}
export interface IShields {
    shieldLevel: Shield;
}
export interface IStageInfo {
    chests: Buffer;
    switches: Buffer;
    items: Buffer;
    rooms: Buffer;
    keys: number;
    map: boolean;
    compass: boolean;
    bigKey: boolean;
    bossKilled: boolean;
    heartTaken: boolean;
    bossIntroWatched: boolean;
}
export declare const enum Magic {
    NONE = 0,
    NORMAL = 1,
    EXTENDED = 2
}
export declare const enum MagicQuantities {
    NONE = 0,
    NORMAL = 16,
    EXTENDED = 32
}
export interface IInventory extends IInventoryFields, IInventoryCounts {
    getItemInSlot(slotId: number): InventoryItem;
    getSlotForItem(item: InventoryItem): number;
    setItemInSlot(item: InventoryItem, slot: number): void;
}
export declare enum InventoryItem {
    TELESCOPE = 32,
    SAIL = 120,
    WIND_WAKER = 34,
    GRAPPLING_HOOK = 37,
    SPOILS_BAG = 36,
    BOOMERANG = 45,
    DEKU_LEAF = 52,
    TINGLE_TUNER = 33,
    PICTO_BOX = 35,
    DELUXE_PICTO_BOX = 38,
    IRON_BOOTS = 41,
    MAGIC_ARMOR = 42,
    BAIT_BAG = 44,
    BOW = 39,
    FI_BOW = 53,
    LIGHT_BOW = 54,
    BOMBS = 49,
    BOTTLE_EMPTY = 80,
    BOTTLE_POTION_RED = 81,
    BOTTLE_POTION_GREEN = 82,
    BOTTLE_POTION_BLUE = 83,
    BOTTLE_ELIXER_HALF = 84,
    BOTTLE_ELIXER_FULL = 85,
    BOTTLE_WATER = 86,
    BOTTLE_FAIRY = 87,
    BOTTLE_FIREFLY = 88,
    BOTTLE_FOREST_WATER = 89,
    DELIVERY_BAG = 48,
    HOOKSHOT = 47,
    SKULL_HAMMER = 51,
    NONE = 255
}
export interface IInventoryCounts {
    arrowCount: number;
    bombCount: number;
    rupeeCount: number;
}
export interface IInventoryFields {
    FIELD_TELESCOPE: boolean;
    FIELD_SAIL: boolean;
    FIELD_WIND_WAKER: boolean;
    FIELD_GRAPPLING_HOOK: boolean;
    FIELD_SPOILS_BAG: boolean;
    FIELD_BOOMERANG: boolean;
    FIELD_DEKU_LEAF: boolean;
    FIELD_TINGLE_TUNER: boolean;
    FIELD_PICTO_BOX: InventoryItem;
    FIELD_IRON_BOOTS: boolean;
    FIELD_MAGIC_ARMOR: boolean;
    FIELD_BAIT_BAG: boolean;
    FIELD_BOW: InventoryItem;
    FIELD_BOMBS: boolean;
    FIELD_BOTTLE1: InventoryItem;
    FIELD_BOTTLE2: InventoryItem;
    FIELD_BOTTLE3: InventoryItem;
    FIELD_BOTTLE4: InventoryItem;
    FIELD_DELIVERY_BAG: boolean;
    FIELD_HOOKSHOT: boolean;
    FIELD_SKULL_HAMMER: boolean;
    spoils_slots: Buffer;
    bait_slots: Buffer;
    delivery_slots: Buffer;
    owned_delivery: Buffer;
    owned_spoils: Buffer;
    owned_bait: Buffer;
    count_spoils: Buffer;
    count_delivery: Buffer;
    count_bait: Buffer;
    owned_items: Buffer;
    rupeeCap: number;
    bombCap: number;
    arrowCap: number;
}
export declare const enum InventorySlots {
    TELESCOPE = 0,
    SAIL = 1,
    WIND_WAKER = 2,
    GRAPPLING_HOOK = 3,
    SPOILS_BAG = 4,
    BOOMERANG = 5,
    DEKU_LEAF = 6,
    TINGLE_TUNER = 7,
    PICTO_BOX = 8,
    IRON_BOOTS = 9,
    MAGIC_ARMOR = 10,
    BAIT_BAG = 11,
    BOW = 12,
    BOMBS = 13,
    BOTTLE1 = 14,
    BOTTLE2 = 15,
    BOTTLE3 = 16,
    BOTTLE4 = 17,
    DELIVERY_BAG = 18,
    HOOKSHOT = 19,
    SKULL_HAMMER = 20
}
export interface IQuestStatus {
    hasTunic: boolean;
    hero_charm: boolean;
    current_hp: number;
    max_hp: number;
    current_mp: number;
    max_mp: number;
    bracelet: number;
    triforce: Buffer;
    pearls: Buffer;
    songs: Buffer;
    pirate_charm: Buffer;
    owned_charts: Buffer;
    opened_charts: Buffer;
    completed_charts: Buffer;
    sectors: Buffer;
    deciphered_triforce: Buffer;
    tingle_statues: Buffer;
}
export interface IGlobalContext {
    current_stage_id: number;
    current_scene_frame: number;
    current_scene_name: string;
    current_room_number: number;
    prev_room_number: number;
    next_scene_name: string;
    next_room_number: number;
    linkPointer: number;
    getSaveDataForCurrentScene(): Buffer;
    writeSaveDataForCurrentScene(buf: Buffer): void;
}
export interface ISaveContext {
    inventory: IInventory;
    questStatus: IQuestStatus;
    shields: IShields;
    swords: ISwords;
    eventFlags: Buffer;
    stage_Live: IStageInfo;
    stage0_Sea: IStageInfo;
    stage1_SeaAlt: IStageInfo;
    stage2_ForsakenFortress: IStageInfo;
    stage3_DRC: IStageInfo;
    stage4_FW: IStageInfo;
    stage5_TOTG: IStageInfo;
    stage6_ET: IStageInfo;
    stage7_WT: IStageInfo;
    stage8_GT: IStageInfo;
    stage9_Hyrule: IStageInfo;
    stageA_ShipInt: IStageInfo;
    stageB_HouseMisc: IStageInfo;
    stageC_CaveInt: IStageInfo;
    stageD_CaveShip: IStageInfo;
    stageE_BlueChu: IStageInfo;
    stageF_TestMaps: IStageInfo;
}
export interface ILink {
    pos: Buffer;
    rot: Buffer;
    matrixData: Buffer;
}
export interface IWWHelper {
    isLoadingZone(): boolean;
    isLinkExists(): boolean;
    isTitleScreen(): boolean;
    isSceneNameValid(): boolean;
    isLinkControllable(): boolean;
    isPaused(): boolean;
    isSceneChange(): boolean;
}
export declare enum WWEvents {
    ON_SAVE_LOADED = "onSaveLoaded",
    ON_SCENE_CHANGE = "onSceneChange",
    ON_ROOM_CHANGE = "onRoomChange",
    ON_LOADING_ZONE = "onLoadingZone"
}
export interface IWWCore extends ICore {
    heap?: Heap;
    global: IGlobalContext;
    link: ILink;
    save: ISaveContext;
    helper: IWWHelper;
}
//# sourceMappingURL=WWAPI.d.ts.map