"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SwordsEquipment = void 0;
const JSONTemplate_1 = require("modloader64_api/JSONTemplate");
class SwordsEquipment extends JSONTemplate_1.JSONTemplate {
    emulator;
    core;
    jsonFields = [
        'swordLevel',
    ];
    constructor(emulator, core) {
        super();
        this.emulator = emulator;
        this.core = core;
    }
    get swordLevel() {
        let bits = this.emulator.rdramReadBits8(0x803C4CBC); //Sword bitfield
        if (bits[4 /* SwordBitMap.MasterFull */] === 1) {
            return 4 /* Sword.MasterFull */;
        }
        else if (bits[5 /* SwordBitMap.MasterHalf */] === 1) {
            return 3 /* Sword.MasterHalf */;
        }
        else if (bits[6 /* SwordBitMap.Master */] === 1) {
            return 2 /* Sword.Master */;
        }
        else if (bits[7 /* SwordBitMap.Hero */] === 1) {
            return 1 /* Sword.Hero */;
        }
        else {
            return 0 /* Sword.NONE */;
        }
    }
    /*   803C4C16,1 - Currently equipped swor
      38 - Hero's Sword
      39 - Master Sword (Powerless)
      3A - Master Sword (Half Power)
      3E - Master Sword (Full Power) */
    set swordLevel(level) {
        let bits = this.emulator.rdramReadBits8(0x803C4CBC);
        switch (level) {
            case 0 /* Sword.NONE */:
                bits[7 /* SwordBitMap.Hero */] = 0;
                bits[6 /* SwordBitMap.Master */] = 0;
                bits[5 /* SwordBitMap.MasterHalf */] = 0;
                bits[4 /* SwordBitMap.MasterFull */] = 0;
                this.emulator.rdramWrite8(0x803C4C16, 0xFF);
                break;
            case 1 /* Sword.Hero */:
                bits[7 /* SwordBitMap.Hero */] = 1;
                bits[6 /* SwordBitMap.Master */] = 0;
                bits[5 /* SwordBitMap.MasterHalf */] = 0;
                bits[4 /* SwordBitMap.MasterFull */] = 0;
                this.emulator.rdramWrite8(0x803C4C16, 0x38);
                break;
            case 2 /* Sword.Master */:
                bits[7 /* SwordBitMap.Hero */] = 1;
                bits[6 /* SwordBitMap.Master */] = 1;
                bits[5 /* SwordBitMap.MasterHalf */] = 0;
                bits[4 /* SwordBitMap.MasterFull */] = 0;
                this.emulator.rdramWrite8(0x803C4C16, 0x39);
                break;
            case 3 /* Sword.MasterHalf */:
                bits[7 /* SwordBitMap.Hero */] = 1;
                bits[6 /* SwordBitMap.Master */] = 1;
                bits[5 /* SwordBitMap.MasterHalf */] = 1;
                bits[4 /* SwordBitMap.MasterFull */] = 0;
                this.emulator.rdramWrite8(0x803C4C16, 0x3A);
                break;
            case 4 /* Sword.MasterFull */:
                bits[7 /* SwordBitMap.Hero */] = 1;
                bits[6 /* SwordBitMap.Master */] = 1;
                bits[5 /* SwordBitMap.MasterHalf */] = 1;
                bits[4 /* SwordBitMap.MasterFull */] = 1;
                this.emulator.rdramWrite8(0x803C4C16, 0x3E);
                break;
        }
        this.emulator.rdramWriteBits8(0x803C4CBC, bits);
    }
}
exports.SwordsEquipment = SwordsEquipment;
//# sourceMappingURL=SwordsEquipment.js.map